@@ -30,9 +30,9 @@ extern void db_del_macro_cmd (void);
 
 extern void db_show_macro (void);
 
-extern int db_exec_macro (char *name);
+extern int db_exec_macro (const char *name);
 
-extern long db_arg_variable (
+extern void db_arg_variable (
 	struct db_variable *vp,
 	db_expr_t *valuep,
 	int flag,
