@@ -94,6 +94,7 @@ boolean_t	db_disasm_16 = FALSE;
 #define	STI	33			/* FP stack */
 #define	X	34			/* extended FP op */
 #define	XA	35			/* for 'fstcw %ax' */
+#define	Iba	36			/* byte immediate, don't print if 0xa */
 
 struct inst {
 	char *	i_name;			/* name */
@@ -749,8 +750,8 @@ struct inst db_inst_table[256] = {
 /*d1*/	{ "",	   TRUE,  LONG,  op2(o1, E),  (char *)db_Grp2 },
 /*d2*/	{ "",	   TRUE,  BYTE,  op2(CL, E),  (char *)db_Grp2 },
 /*d3*/	{ "",	   TRUE,  LONG,  op2(CL, E),  (char *)db_Grp2 },
-/*d4*/	{ "aam",   TRUE,  NONE,  0,	      0 },
-/*d5*/	{ "aad",   TRUE,  NONE,  0,	      0 },
+/*d4*/	{ "aam",   FALSE, NONE,  op1(Iba),    0 },
+/*d5*/	{ "aad",   FALSE, NONE,  op1(Iba),    0 },
 /*d6*/	{ "",      FALSE, NONE,  0,	      0 },
 /*d7*/	{ "xlat",  FALSE, BYTE,  op1(BX),     0 },
 
@@ -949,10 +950,10 @@ db_read_address(
 
 void
 db_print_address(
-	char *		seg,
-	int		size,
-	struct i_addr	*addrp,
-	task_t		task)
+	const char *		seg,
+	int			size,
+	const struct i_addr	*addrp,
+	task_t			task)
 {
 	if (addrp->is_reg) {
 	    db_printf("%s", db_reg[size][addrp->disp]);
@@ -985,7 +986,7 @@ db_disasm_esc(
 	int		inst,
 	int		short_addr,
 	int		size,
-	char *		seg,
+	const char *	seg,
 	task_t		task)
 {
 	int		regmodrm;
@@ -1333,6 +1334,12 @@ db_disasm(
 		    db_printf("$%#n", imm);
 		    break;
 
+		case Iba:
+		    get_value_inc(imm, loc, 1, FALSE, task);
+		    if (imm != 0x0a)
+			db_printf("$%#r", imm);
+		    break;
+
 		case Ibs:
 		    get_value_inc(imm, loc, 1, TRUE, task);	/* signed */
 		    db_printf("$%#r", imm);
