@@ -24,27 +24,29 @@
  * the rights to redistribute these changes.
  */
 
+#include <stddef.h>
 #include <string.h>
 
 #include <mach/std_types.h>
 #include <mach/kern_return.h>
 #include <mach/thread_status.h>
 #include <mach/exec/exec.h>
+#include <mach/xen.h>
 
 #include "vm_param.h"
 #include <kern/counters.h>
 #include <kern/debug.h>
-#include <kern/mach_param.h>
 #include <kern/thread.h>
 #include <kern/sched_prim.h>
+#include <kern/slab.h>
 #include <vm/vm_kern.h>
 #include <vm/pmap.h>
 
 #include <i386/thread.h>
 #include <i386/proc_reg.h>
 #include <i386/seg.h>
-#include <i386/tss.h>
 #include <i386/user_ldt.h>
+#include <i386/db_interface.h>
 #include <i386/fpu.h>
 #include "eflags.h"
 #include "gdt.h"
@@ -52,18 +54,13 @@
 #include "ktss.h"
 #include "pcb.h"
 
+#include <machine/tss.h>
+
 #if	NCPUS > 1
 #include <i386/mp_desc.h>
 #endif
 
-extern thread_t	Switch_context();
-extern void	Thread_continue();
-
-extern iopb_tss_t	iopb_create();
-extern void		iopb_destroy();
-extern void		user_ldt_free();
-
-zone_t		pcb_zone;
+struct kmem_cache	pcb_cache;
 
 vm_offset_t	kernel_stack[NCPUS];	/* top of active_stack */
 
@@ -73,10 +70,10 @@ vm_offset_t	kernel_stack[NCPUS];	/* top of active_stack */
  *	Attach a kernel stack to a thread.
  */
 
-void stack_attach(thread, stack, continuation)
-	register thread_t thread;
-	register vm_offset_t stack;
-	void (*continuation)();
+void stack_attach(
+	thread_t 	thread,
+	vm_offset_t 	stack,
+	void 		(*continuation)(thread_t))
 {
 	counter(if (++c_stacks_current > c_stacks_max)
 			c_stacks_max = c_stacks_current);
@@ -91,9 +88,10 @@ void stack_attach(thread, stack, continuation)
 	 *	This function will not return normally,
 	 *	so we don`t have to worry about a return address.
 	 */
-	STACK_IKS(stack)->k_eip = (int) Thread_continue;
-	STACK_IKS(stack)->k_ebx = (int) continuation;
-	STACK_IKS(stack)->k_esp = (int) STACK_IEL(stack);
+	STACK_IKS(stack)->k_eip = (long) Thread_continue;
+	STACK_IKS(stack)->k_ebx = (long) continuation;
+	STACK_IKS(stack)->k_esp = (long) STACK_IEL(stack);
+	STACK_IKS(stack)->k_ebp = (long) 0;
 
 	/*
 	 *	Point top of kernel stack to user`s registers.
@@ -107,10 +105,9 @@ void stack_attach(thread, stack, continuation)
  *	Detaches a kernel stack from a thread, returning the old stack.
  */
 
-vm_offset_t stack_detach(thread)
-	register thread_t	thread;
+vm_offset_t stack_detach(thread_t thread)
 {
-	register vm_offset_t	stack;
+	vm_offset_t	stack;
 
 	counter(if (--c_stacks_current < c_stacks_min)
 			c_stacks_min = c_stacks_current);
@@ -126,18 +123,16 @@ vm_offset_t stack_detach(thread)
 #define	curr_ktss(mycpu)	(mp_ktss[mycpu])
 #else
 #define	curr_gdt(mycpu)		((void)(mycpu), gdt)
-#define	curr_ktss(mycpu)	((void)(mycpu), &ktss)
+#define	curr_ktss(mycpu)	((void)(mycpu), (struct task_tss *)&ktss)
 #endif
 
 #define	gdt_desc_p(mycpu,sel) \
 	((struct real_descriptor *)&curr_gdt(mycpu)[sel_idx(sel)])
 
-void switch_ktss(pcb)
-	register pcb_t	pcb;
+void switch_ktss(pcb_t pcb)
 {
 	int			mycpu = cpu_number();
     {
-	register iopb_tss_t	tss = pcb->ims.io_tss;
 	vm_offset_t		pcb_stack_top;
 
 	/*
@@ -150,55 +145,70 @@ void switch_ktss(pcb)
 	 */
 
 	pcb_stack_top = (pcb->iss.efl & EFL_VM)
-			? (int) (&pcb->iss + 1)
-			: (int) (&pcb->iss.v86_segs);
+			? (long) (&pcb->iss + 1)
+			: (long) (&pcb->iss.v86_segs);
 
-	if (tss == 0) {
-	    /*
-	     *	No per-thread IO permissions.
-	     *	Use standard kernel TSS.
-	     */
-	    if (!(gdt_desc_p(mycpu,KERNEL_TSS)->access & ACC_TSS_BUSY))
-		set_tr(KERNEL_TSS);
-	    curr_ktss(mycpu)->esp0 = pcb_stack_top;
-	}
-	else {
-	    /*
-	     * Set the IO permissions.  Use this thread`s TSS.
-	     */
-	    *gdt_desc_p(mycpu,USER_TSS)
-	    	= *(struct real_descriptor *)tss->iopb_desc;
-	    tss->tss.esp0 = pcb_stack_top;
-	    set_tr(USER_TSS);
-	    gdt_desc_p(mycpu,KERNEL_TSS)->access &= ~ ACC_TSS_BUSY;
-	}
+#ifdef	MACH_RING1
+	/* No IO mask here */
+	if (hyp_stack_switch(KERNEL_DS, pcb_stack_top))
+		panic("stack_switch");
+#else	/* MACH_RING1 */
+	curr_ktss(mycpu)->tss.esp0 = pcb_stack_top;
+#endif	/* MACH_RING1 */
     }
 
     {
-	register user_ldt_t	ldt = pcb->ims.ldt;
+	user_ldt_t	tldt = pcb->ims.ldt;
 	/*
 	 * Set the thread`s LDT.
 	 */
-	if (ldt == 0) {
+	if (tldt == 0) {
 	    /*
 	     * Use system LDT.
 	     */
+#ifdef	MACH_PV_DESCRIPTORS
+	    hyp_set_ldt(&ldt, LDTSZ);
+#else	/* MACH_PV_DESCRIPTORS */
 	    set_ldt(KERNEL_LDT);
+#endif	/* MACH_PV_DESCRIPTORS */
 	}
 	else {
 	    /*
 	     * Thread has its own LDT.
 	     */
-	    *gdt_desc_p(mycpu,USER_LDT) = ldt->desc;
+#ifdef	MACH_PV_DESCRIPTORS
+	    hyp_set_ldt(tldt->ldt,
+	    		(tldt->desc.limit_low|(tldt->desc.limit_high<<16)) /
+				sizeof(struct real_descriptor));
+#else	/* MACH_PV_DESCRIPTORS */
+	    *gdt_desc_p(mycpu,USER_LDT) = tldt->desc;
 	    set_ldt(USER_LDT);
+#endif	/* MACH_PV_DESCRIPTORS */
 	}
     }
 
+#ifdef	MACH_PV_DESCRIPTORS
+    {
+	int i;
+	for (i=0; i < USER_GDT_SLOTS; i++) {
+	    if (memcmp(gdt_desc_p (mycpu, USER_GDT + (i << 3)),
+		&pcb->ims.user_gdt[i], sizeof pcb->ims.user_gdt[i])) {
+		if (hyp_do_update_descriptor(kv_to_ma(gdt_desc_p (mycpu, USER_GDT + (i << 3))),
+			*(uint64_t *) &pcb->ims.user_gdt[i]))
+		    panic("couldn't set user gdt %d\n",i);
+	    }
+	}
+    }
+#else /* MACH_PV_DESCRIPTORS */
+
     /* Copy in the per-thread GDT slots.  No reloading is necessary
        because just restoring the segment registers on the way back to
        user mode reloads the shadow registers from the in-memory GDT.  */
     memcpy (gdt_desc_p (mycpu, USER_GDT),
         pcb->ims.user_gdt, sizeof pcb->ims.user_gdt);
+#endif /* MACH_PV_DESCRIPTORS */
+
+	db_load_context(pcb);
 
 	/*
 	 * Load the floating-point context, if necessary.
@@ -207,18 +217,36 @@ void switch_ktss(pcb)
 
 }
 
+/* If NEW_IOPB is not null, the SIZE denotes the number of bytes in
+   the new bitmap.  Expects iopb_lock to be held.  */
+void
+update_ktss_iopb (unsigned char *new_iopb, io_port_t size)
+{
+  struct task_tss *tss = curr_ktss (cpu_number ());
+
+  if (new_iopb && size > 0)
+    {
+      tss->tss.io_bit_map_offset
+       = offsetof (struct task_tss, barrier) - size;
+      memcpy (((char *) tss) + tss->tss.io_bit_map_offset,
+             new_iopb, size);
+    }
+  else
+    tss->tss.io_bit_map_offset = IOPB_INVAL;
+}
+
 /*
  *	stack_handoff:
  *
  *	Move the current thread's kernel stack to the new thread.
  */
 
-void stack_handoff(old, new)
-	register thread_t	old;
-	register thread_t	new;
+void stack_handoff(
+	thread_t	old,
+	thread_t	new)
 {
-	register int		mycpu = cpu_number();
-	register vm_offset_t	stack;
+	int		mycpu = cpu_number();
+	vm_offset_t	stack;
 
 	/*
 	 *	Save FP registers if in use.
@@ -236,6 +264,19 @@ void stack_handoff(old, new)
 				     old, mycpu);
 		PMAP_ACTIVATE_USER(vm_map_pmap(new_task->map),
 				   new, mycpu);
+
+		simple_lock (&new_task->machine.iopb_lock);
+#if NCPUS>1
+#warning SMP support missing (avoid races with io_perm_modify).
+#else
+		/* This optimization only works on a single processor
+		   machine, where old_task's iopb can not change while
+		   we are switching.  */
+		if (old_task->machine.iopb || new_task->machine.iopb)
+#endif
+		  update_ktss_iopb (new_task->machine.iopb,
+				    new_task->machine.iopb_size);
+		simple_unlock (&new_task->machine.iopb_lock);
 	}
     }
 
@@ -264,8 +305,7 @@ void stack_handoff(old, new)
 /*
  * Switch to the first thread on a CPU.
  */
-void load_context(new)
-	register thread_t	new;
+void load_context(thread_t new)
 {
 	switch_ktss(new->pcb);
 	Load_context(new);
@@ -276,10 +316,10 @@ void load_context(new)
  * Save the old thread`s kernel state or continuation,
  * and return it.
  */
-thread_t switch_context(old, continuation, new)
-	register thread_t	old;
-	void (*continuation)();
-	register thread_t	new;
+thread_t switch_context(
+	thread_t	old,
+	void 		(*continuation)(),
+	thread_t	new)
 {
 	/*
 	 *	Save FP registers if in use.
@@ -298,6 +338,19 @@ thread_t switch_context(old, continuation, new)
 				     old, mycpu);
 		PMAP_ACTIVATE_USER(vm_map_pmap(new_task->map),
 				   new, mycpu);
+
+		simple_lock (&new_task->machine.iopb_lock);
+#if NCPUS>1
+#warning SMP support missing (avoid races with io_perm_modify).
+#else
+		/* This optimization only works on a single processor
+		   machine, where old_task's iopb can not change while
+		   we are switching.  */
+		if (old_task->machine.iopb || new_task->machine.iopb)
+#endif
+		  update_ktss_iopb (new_task->machine.iopb,
+				    new_task->machine.iopb_size);
+		simple_unlock (&new_task->machine.iopb_lock);
 	}
     }
 
@@ -309,23 +362,19 @@ thread_t switch_context(old, continuation, new)
 	return Switch_context(old, continuation, new);
 }
 
-void pcb_module_init()
+void pcb_module_init(void)
 {
-	pcb_zone = zinit(sizeof(struct pcb),
-			 THREAD_MAX * sizeof(struct pcb),
-			 THREAD_CHUNK * sizeof(struct pcb),
-			 0, "i386 pcb state");
+	kmem_cache_init(&pcb_cache, "pcb", sizeof(struct pcb), 0,
+			NULL, NULL, NULL, 0);
 
 	fpu_module_init();
-	iopb_init();
 }
 
-void pcb_init(thread)
-	register thread_t	thread;
+void pcb_init(thread_t thread)
 {
-	register pcb_t		pcb;
+	pcb_t		pcb;
 
-	pcb = (pcb_t) zalloc(pcb_zone);
+	pcb = (pcb_t) kmem_cache_alloc(&pcb_cache);
 	if (pcb == 0)
 		panic("pcb_init");
 
@@ -353,21 +402,18 @@ void pcb_init(thread)
 	thread->pcb = pcb;
 }
 
-void pcb_terminate(thread)
-	register thread_t	thread;
+void pcb_terminate(thread_t thread)
 {
-	register pcb_t		pcb = thread->pcb;
+	pcb_t		pcb = thread->pcb;
 
 	counter(if (--c_threads_current < c_threads_min)
 			c_threads_min = c_threads_current);
 
-	if (pcb->ims.io_tss != 0)
-		iopb_destroy(pcb->ims.io_tss);
 	if (pcb->ims.ifps != 0)
 		fp_free(pcb->ims.ifps);
 	if (pcb->ims.ldt != 0)
 		user_ldt_free(pcb->ims.ldt);
-	zfree(pcb_zone, (vm_offset_t) pcb);
+	kmem_cache_free(&pcb_cache, (vm_offset_t) pcb);
 	thread->pcb = 0;
 }
 
@@ -378,7 +424,7 @@ void pcb_terminate(thread)
  */
 
 void pcb_collect(thread)
-	thread_t thread;
+	const thread_t thread;
 {
 }
 
@@ -389,18 +435,18 @@ void pcb_collect(thread)
  *	Set the status of the specified thread.
  */
 
-kern_return_t thread_setstatus(thread, flavor, tstate, count)
-	thread_t		thread;
-	int			flavor;
-	thread_state_t		tstate;
-	unsigned int		count;
+kern_return_t thread_setstatus(
+	thread_t		thread,
+	int			flavor,
+	thread_state_t		tstate,
+	unsigned int		count)
 {
 	switch (flavor) {
 	    case i386_THREAD_STATE:
 	    case i386_REGS_SEGS_STATE:
 	    {
-		register struct i386_thread_state	*state;
-		register struct i386_saved_state	*saved_state;
+		struct i386_thread_state	*state;
+		struct i386_saved_state	*saved_state;
 
 		if (count < i386_THREAD_STATE_COUNT) {
 		    return(KERN_INVALID_ARGUMENT);
@@ -515,8 +561,7 @@ kern_return_t thread_setstatus(thread, flavor, tstate, count)
 	     * Temporary - replace by i386_io_map
 	     */
 	    case i386_ISA_PORT_MAP_STATE: {
-		register struct i386_isa_port_map_state *state;
-		register iopb_tss_t	tss;
+		//register struct i386_isa_port_map_state *state;
 
 		if (count < i386_ISA_PORT_MAP_STATE_COUNT)
 			return(KERN_INVALID_ARGUMENT);
@@ -543,7 +588,7 @@ kern_return_t thread_setstatus(thread, flavor, tstate, count)
 
 	    case i386_V86_ASSIST_STATE:
 	    {
-		register struct i386_v86_assist_state *state;
+		struct i386_v86_assist_state *state;
 		vm_offset_t	int_table;
 		int		int_count;
 
@@ -568,6 +613,21 @@ kern_return_t thread_setstatus(thread, flavor, tstate, count)
 		break;
 	    }
 
+	    case i386_DEBUG_STATE:
+	    {
+		struct i386_debug_state *state;
+		kern_return_t ret;
+
+		if (count < i386_DEBUG_STATE_COUNT)
+		    return KERN_INVALID_ARGUMENT;
+
+		state = (struct i386_debug_state *) tstate;
+		ret = db_set_debug_state(thread->pcb, state);
+		if (ret)
+			return ret;
+		break;
+	    }
+
 	    default:
 		return(KERN_INVALID_ARGUMENT);
 	}
@@ -581,11 +641,11 @@ kern_return_t thread_setstatus(thread, flavor, tstate, count)
  *	Get the status of the specified thread.
  */
 
-kern_return_t thread_getstatus(thread, flavor, tstate, count)
-	register thread_t	thread;
-	int			flavor;
-	thread_state_t		tstate;	/* pointer to OUT array */
-	unsigned int		*count;		/* IN/OUT */
+kern_return_t thread_getstatus(
+	thread_t		thread,
+	int			flavor,
+	thread_state_t		tstate,	/* pointer to OUT array */
+	unsigned int		*count)		/* IN/OUT */
 {
 	switch (flavor)  {
 	    case THREAD_STATE_FLAVOR_LIST:
@@ -601,8 +661,8 @@ kern_return_t thread_getstatus(thread, flavor, tstate, count)
 	    case i386_THREAD_STATE:
 	    case i386_REGS_SEGS_STATE:
 	    {
-		register struct i386_thread_state	*state;
-		register struct i386_saved_state	*saved_state;
+		struct i386_thread_state	*state;
+		struct i386_saved_state	*saved_state;
 
 		if (*count < i386_THREAD_STATE_COUNT)
 		    return(KERN_INVALID_ARGUMENT);
@@ -672,33 +732,21 @@ kern_return_t thread_getstatus(thread, flavor, tstate, count)
 	     * Temporary - replace by i386_io_map
 	     */
 	    case i386_ISA_PORT_MAP_STATE: {
-		register struct i386_isa_port_map_state *state;
-		register iopb_tss_t tss;
+		struct i386_isa_port_map_state *state;
 
 		if (*count < i386_ISA_PORT_MAP_STATE_COUNT)
 			return(KERN_INVALID_ARGUMENT);
 
 		state = (struct i386_isa_port_map_state *) tstate;
-		tss = thread->pcb->ims.io_tss;
 
-		if (tss == 0) {
-		    int i;
-
-		    /*
-		     *	The thread has no ktss, so no IO permissions.
-		     */
-
-		    for (i = 0; i < sizeof state->pm; i++)
-			state->pm[i] = 0xff;
-		} else {
-		    /*
-		     *	The thread has its own ktss.
-		     */
-
-		    memcpy(state->pm,
-			   tss->bitmap,
-			   sizeof state->pm);
-		}
+		simple_lock (&thread->task->machine.iopb_lock);
+		if (thread->task->machine.iopb == 0)
+		  memset (state->pm, 0xff, sizeof state->pm);
+		else
+		  memcpy(state->pm,
+			 thread->task->machine.iopb,
+			 sizeof state->pm);
+		simple_unlock (&thread->task->machine.iopb_lock);
 
 		*count = i386_ISA_PORT_MAP_STATE_COUNT;
 		break;
@@ -706,7 +754,7 @@ kern_return_t thread_getstatus(thread, flavor, tstate, count)
 
 	    case i386_V86_ASSIST_STATE:
 	    {
-		register struct i386_v86_assist_state *state;
+		struct i386_v86_assist_state *state;
 
 		if (*count < i386_V86_ASSIST_STATE_COUNT)
 		    return KERN_INVALID_ARGUMENT;
@@ -719,6 +767,20 @@ kern_return_t thread_getstatus(thread, flavor, tstate, count)
 		break;
 	    }
 
+	    case i386_DEBUG_STATE:
+	    {
+		struct i386_debug_state *state;
+
+		if (*count < i386_DEBUG_STATE_COUNT)
+		    return KERN_INVALID_ARGUMENT;
+
+		state = (struct i386_debug_state *) tstate;
+		db_get_debug_state(thread->pcb, state);
+
+		*count = i386_DEBUG_STATE_COUNT;
+		break;
+	    }
+
 	    default:
 		return(KERN_INVALID_ARGUMENT);
 	}
@@ -731,24 +793,22 @@ kern_return_t thread_getstatus(thread, flavor, tstate, count)
  * will make the thread return 'retval' from a syscall.
  */
 void
-thread_set_syscall_return(thread, retval)
-	thread_t	thread;
-	kern_return_t	retval;
+thread_set_syscall_return(
+	thread_t	thread,
+	kern_return_t	retval)
 {
 	thread->pcb->iss.eax = retval;
 }
 
-
 /*
- * Return prefered address of user stack.
+ * Return preferred address of user stack.
  * Always returns low address.  If stack grows up,
  * the stack grows away from this address;
  * if stack grows down, the stack grows towards this
  * address.
  */
 vm_offset_t
-user_stack_low(stack_size)
-	vm_size_t	stack_size;
+user_stack_low(vm_size_t stack_size)
 {
 	return (VM_MAX_ADDRESS - stack_size);
 }
@@ -760,17 +820,17 @@ vm_offset_t
 set_user_regs(stack_base, stack_size, exec_info, arg_size)
 	vm_offset_t	stack_base;	/* low address */
 	vm_offset_t	stack_size;
-	struct exec_info *exec_info;
+	const struct exec_info *exec_info;
 	vm_size_t	arg_size;
 {
 	vm_offset_t	arg_addr;
-	register struct i386_saved_state *saved_state;
+	struct i386_saved_state *saved_state;
 
 	arg_size = (arg_size + sizeof(int) - 1) & ~(sizeof(int)-1);
 	arg_addr = stack_base + stack_size - arg_size;
 
 	saved_state = USER_REGS(current_thread());
-	saved_state->uesp = (int)arg_addr;
+	saved_state->uesp = (long)arg_addr;
 	saved_state->eip = exec_info->entry;
 
 	return (arg_addr);
