@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (c) 1994 The University of Utah and
  * the Computer Systems Laboratory at the University of Utah (CSL).
  * All rights reserved.
@@ -32,6 +32,10 @@
 
 #define PIC_INT_BASE 0x40
 
-#include "idt-gen.h"
+#include <i386/idt-gen.h>
 
-#endif _I386AT_IDT_
+#ifndef __ASSEMBLER__
+extern void idt_init (void);
+#endif /* __ASSEMBLER__ */
+
+#endif /* _I386AT_IDT_ */
