@@ -39,7 +39,7 @@
 #include <mach/machine/vm_types.h>
 
 
-typedef natural_t mach_port_t;
+typedef vm_offset_t mach_port_t;
 typedef mach_port_t *mach_port_array_t;
 typedef int *rpc_signature_info_t;
 
@@ -137,6 +137,8 @@ typedef struct mach_port_status {
 /*
  *  Compatibility definitions, for code written
  *  before there was an mps_seqno field.
+ *
+ *  XXX: Remove this before releasing Gnumach 1.6.
  */
 
 typedef struct old_mach_port_status {
@@ -150,36 +152,4 @@ typedef struct old_mach_port_status {
 /*boolean_t*/natural_t		mps_nsrequest;	/* no-senders requested? */
 } old_mach_port_status_t;
 
-
-/* Definitions for the old IPC interface. */
-
-#if	MACH_IPC_COMPAT
-
-typedef integer_t	port_name_t;		/* A capability's name */
-typedef port_name_t	port_set_name_t;	/* Descriptive alias */
-typedef port_name_t	*port_name_array_t;
-
-typedef integer_t	port_type_t;		/* What kind of capability? */
-typedef port_type_t	*port_type_array_t;
-
-	/* Values for port_type_t */
-
-#define PORT_TYPE_NONE		0		/* No rights */
-#define PORT_TYPE_SEND		1		/* Send rights */
-#define PORT_TYPE_RECEIVE	3		/* obsolete */
-#define PORT_TYPE_OWN		5		/* obsolete */
-#define PORT_TYPE_RECEIVE_OWN	7		/* Send, receive, ownership */
-#define PORT_TYPE_SET		9		/* Set ownership */
-#define PORT_TYPE_LAST		10		/* Last assigned */
-
-typedef	port_name_t	port_t;			/* Port with send rights */
-typedef	port_t		port_rcv_t;		/* Port with receive rights */
-typedef	port_t		port_own_t;		/* Port with ownership rights */
-typedef	port_t		port_all_t;		/* Port with receive and ownership */
-typedef	port_t		*port_array_t;
-
-#define PORT_NULL	((port_name_t) 0)	/* Used to denote no port; legal value */
-
-#endif	/* MACH_IPC_COMPAT */
-
 #endif	/* _MACH_PORT_H_ */
