@@ -38,11 +38,13 @@
 #ifndef	_IPC_IPC_ENTRY_H_
 #define _IPC_IPC_ENTRY_H_
 
+#include <mach/mach_types.h>
 #include <mach/port.h>
 #include <mach/kern_return.h>
-#include <kern/zalloc.h>
+#include <kern/slab.h>
 #include <ipc/port.h>
 #include <ipc/ipc_table.h>
+#include <ipc/ipc_types.h>
 
 /*
  *	Spaces hold capabilities for ipc_object_t's (ports and port sets).
@@ -127,32 +129,33 @@ typedef struct ipc_tree_entry {
 #define	ite_request	ite_entry.ie_request
 #define	ite_next	ite_entry.hash.tree
 
-extern zone_t ipc_tree_entry_zone;
+extern struct kmem_cache ipc_tree_entry_cache;
 
-#define ite_alloc()	((ipc_tree_entry_t) zalloc(ipc_tree_entry_zone))
-#define	ite_free(ite)	zfree(ipc_tree_entry_zone, (vm_offset_t) (ite))
+#define ite_alloc()	((ipc_tree_entry_t) kmem_cache_alloc(&ipc_tree_entry_cache))
+#define	ite_free(ite)	kmem_cache_free(&ipc_tree_entry_cache, (vm_offset_t) (ite))
 
 
 extern ipc_entry_t
-ipc_entry_lookup(/* ipc_space_t space, mach_port_t name */);
+ipc_entry_lookup(ipc_space_t space, mach_port_t name);
 
 extern kern_return_t
-ipc_entry_get(/* ipc_space_t space,
-		 mach_port_t *namep, ipc_entry_t *entryp */);
+ipc_entry_get(ipc_space_t space, mach_port_t *namep, ipc_entry_t *entryp);
 
 extern kern_return_t
-ipc_entry_alloc(/* ipc_space_t space,
-		   mach_port_t *namep, ipc_entry_t *entryp */);
+ipc_entry_alloc(ipc_space_t space, mach_port_t *namep, ipc_entry_t *entryp);
 
 extern kern_return_t
-ipc_entry_alloc_name(/* ipc_space_t space, mach_port_t name,
-			ipc_entry_t *entryp */);
+ipc_entry_alloc_name(ipc_space_t space, mach_port_t name, ipc_entry_t *entryp);
 
 extern void
-ipc_entry_dealloc(/* ipc_space_t space, mach_port_t name,
-		     ipc_entry_t entry */);
+ipc_entry_dealloc(ipc_space_t space, mach_port_t name, ipc_entry_t entry);
 
 extern kern_return_t
-ipc_entry_grow_table(/* ipc_space_t space */);
+ipc_entry_grow_table(ipc_space_t space);
+
+ipc_entry_t
+db_ipc_object_by_name(
+	task_t		task,
+	mach_port_t	name);
 
 #endif	/* _IPC_IPC_ENTRY_H_ */
