@@ -37,16 +37,11 @@
 
 #include <mach/boolean.h>
 
-#ifdef	lint
-boolean_t	NEVER;
-boolean_t	ALWAYS;
-#else	/* lint */
 #define		NEVER		FALSE
 #define		ALWAYS		TRUE
-#endif	/* lint */
 
-#define		MACRO_BEGIN	do {
-#define		MACRO_END	} while (NEVER)
+#define		MACRO_BEGIN	({
+#define		MACRO_END	})
 
 #define		MACRO_RETURN	if (ALWAYS) return
 
