@@ -39,7 +39,10 @@
  *	FILE *fd;
  *	char *format;
  *	{
+ *	va_list listp;
+ *	va_start(listp, fmt);
  *	_doprnt(format, &args, fd);
+ *	va_end(listp);
  *	}
  *
  *  would suffice.  (This example does not handle the fprintf's "return
@@ -114,6 +117,7 @@
  */
 
 #include <string.h>
+#include <device/cons.h>
 #include <kern/printf.h>
 #include <mach/boolean.h>
 #include <kern/lock.h>
@@ -126,13 +130,13 @@
 
 
 void printnum(
-	register unsigned long	u,
-	register int		base,
+	unsigned long		u,
+	int			base,
 	void			(*putc)( char, vm_offset_t ),
 	vm_offset_t		putc_arg)
 {
 	char	buf[MAXBUF];	/* build number here */
-	register char *	p = &buf[MAXBUF-1];
+	char *	p = &buf[MAXBUF-1];
 	static char digs[] = "0123456789abcdef";
 
 	do {
@@ -153,7 +157,7 @@ boolean_t	_doprnt_truncates = FALSE;
 boolean_t	_doprnt_lock_initialized = FALSE;
 decl_simple_lock_data(,_doprnt_lock)
 
-void printf_init()
+void printf_init(void)
 {
 	if (!_doprnt_lock_initialized)
 	{
@@ -163,8 +167,8 @@ void printf_init()
 }
 
 void _doprnt(
-	register const char *fmt,
-	va_list		*argp,
+	const char 	*fmt,
+	va_list		argp,
 					/* character output routine */
  	void		(*putc)( char, vm_offset_t),
 	int		radix,		/* default radix - for '%r' */
@@ -180,7 +184,7 @@ void _doprnt(
 	int		sign_char;
 	boolean_t	altfmt, truncate;
 	int		base;
-	register char	c;
+	char		c;
 
 	printf_init();
 
@@ -189,7 +193,7 @@ void _doprnt(
 	simple_lock(&_doprnt_lock);
 #else
 	{
-		register int i = 0;
+		int i = 0;
 		while (i < 1*1024*1024) {
 			if (simple_lock_try(&_doprnt_lock))
 				break;
@@ -247,7 +251,7 @@ void _doprnt(
 		}
 	    }
 	    else if (c == '*') {
-		length = va_arg(*argp, int);
+		length = va_arg(argp, int);
 		c = *++fmt;
 		if (length < 0) {
 		    ladjust = !ladjust;
@@ -265,7 +269,7 @@ void _doprnt(
 		    }
 		}
 		else if (c == '*') {
-		    prec = va_arg(*argp, int);
+		    prec = va_arg(argp, int);
 		    c = *++fmt;
 		}
 	    }
@@ -279,12 +283,12 @@ void _doprnt(
 		case 'b':
 		case 'B':
 		{
-		    register char *p;
-		    boolean_t	  any;
-		    register int  i;
+		    char 	*p;
+		    boolean_t	any;
+		    int  	i;
 
-		    u = va_arg(*argp, unsigned long);
-		    p = va_arg(*argp, char *);
+		    u = va_arg(argp, unsigned long);
+		    p = va_arg(argp, char *);
 		    base = *p++;
 		    printnum(u, base, putc, putc_arg);
 
@@ -298,7 +302,7 @@ void _doprnt(
 			    /*
 			     * Bit field
 			     */
-			    register int j;
+			    int j;
 			    if (any)
 				(*putc)(',', putc_arg);
 			    else {
@@ -332,19 +336,19 @@ void _doprnt(
 		}
 
 		case 'c':
-		    c = va_arg(*argp, int);
+		    c = va_arg(argp, int);
 		    (*putc)(c, putc_arg);
 		    break;
 
 		case 's':
 		{
-		    register char *p;
-		    register char *p2;
+		    char *p;
+		    char *p2;
 
 		    if (prec == -1)
 			prec = 0x7fffffff;	/* MAXINT */
 
-		    p = va_arg(*argp, char *);
+		    p = va_arg(argp, char *);
 
 		    if (p == (char *)0)
 			p = "";
@@ -427,7 +431,7 @@ void _doprnt(
 		    goto print_unsigned;
 
 		print_signed:
-		    n = va_arg(*argp, long);
+		    n = va_arg(argp, long);
 		    if (n >= 0) {
 			u = n;
 			sign_char = plus_sign;
@@ -439,13 +443,13 @@ void _doprnt(
 		    goto print_num;
 
 		print_unsigned:
-		    u = va_arg(*argp, unsigned long);
+		    u = va_arg(argp, unsigned long);
 		    goto print_num;
 
 		print_num:
 		{
 		    char	buf[MAXBUF];	/* build number here */
-		    register char *	p = &buf[MAXBUF-1];
+		    char *	p = &buf[MAXBUF-1];
 		    static char digits[] = "0123456789abcdef";
 		    char *prefix = 0;
 
@@ -510,11 +514,10 @@ void _doprnt(
 /*
  * Printing (to console)
  */
-extern void	cnputc( char, /*not really*/vm_offset_t);
 
 int vprintf(const char *fmt, va_list listp)
 {
-	_doprnt(fmt, &listp, cnputc, 16, 0);
+	_doprnt(fmt, listp, (void (*)( char, vm_offset_t)) cnputc, 16, 0);
 	return 0;
 }
 
@@ -537,7 +540,7 @@ int	indent = 0;
 void iprintf(const char *fmt, ...)
 {
 	va_list	listp;
-	register int i;
+	int i;
 
 	for (i = indent; i > 0; ){
 	    if (i >= 8) {
@@ -550,7 +553,7 @@ void iprintf(const char *fmt, ...)
 	    }
 	}
 	va_start(listp, fmt);
-	_doprnt(fmt, &listp, cnputc, 16, 0);
+	_doprnt(fmt, listp, (void (*)( char, vm_offset_t)) cnputc, 16, 0);
 	va_end(listp);
 }
 
@@ -564,8 +567,8 @@ sputc(
 	char		c,
 	vm_offset_t	arg)
 {
-	register char	**bufp = (char **) arg;
-	register char	*p = *bufp;
+	char	**bufp = (char **) arg;
+	char	*p = *bufp;
 	*p++ = c;
 	*bufp = p;
 }
@@ -577,7 +580,7 @@ sprintf(char *buf, const char *fmt, ...)
 	char	*start = buf;
 
 	va_start(listp, fmt);
-	_doprnt(fmt, &listp, sputc, 16, (vm_offset_t)&buf);
+	_doprnt(fmt, listp, sputc, 16, (vm_offset_t)&buf);
 	va_end(listp);
 
 	*buf = 0;
@@ -601,24 +604,34 @@ snputc(char c, vm_offset_t arg)
 }
 
 int
-vsnprintf(char *buf, int size, const char *fmt, va_list args)
+vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 {
   struct vsnprintf_cookie cookie
     = { .buf = buf, .index = 0, .max_len = size };
 
-  _doprnt (fmt, &args, snputc, 16, (vm_offset_t)&cookie);
+  _doprnt (fmt, args, snputc, 16, (vm_offset_t)&cookie);
   cookie.buf[cookie.index] = '\0';
 
   return cookie.index;
 }
 
-
-void safe_gets(str, maxlen)
-	char *str;
-	int  maxlen;
+int
+snprintf(char *buf, size_t size, const char *fmt, ...)
 {
-	register char *lp;
-	register int c;
+	int written;
+	va_list	listp;
+	va_start(listp, fmt);
+	written = vsnprintf(buf, size, fmt, listp);
+	va_end(listp);
+	return written;
+}
+
+void safe_gets(
+	char *str,
+	int  maxlen)
+{
+	char *lp;
+	int c;
 	char *strmax = str + maxlen - 1; /* allow space for trailing 0 */
 
 	lp = str;
