@@ -39,19 +39,32 @@
 #include <mach/time_value.h>
 #include <mach/mach_param.h>
 #include <mach/task_info.h>
+#include <mach_debug/mach_debug_types.h>
 #include <kern/kern_types.h>
 #include <kern/lock.h>
 #include <kern/queue.h>
 #include <kern/pc_sample.h>
 #include <kern/processor.h>
 #include <kern/syscall_emulation.h>
-#include <vm/vm_map.h>
+#include <vm/vm_types.h>
+#include <machine/task.h>
+
+/*
+ * Task name buffer size.  The size is chosen so that struct task fits
+ * into three cache lines.  The size of a cache line on a typical CPU
+ * is 64 bytes.
+ */
+#define TASK_NAME_SIZE 32
 
 struct task {
 	/* Synchronization/destruction information */
 	decl_simple_lock_data(,lock)	/* Task's lock */
 	int		ref_count;	/* Number of references to me */
-	boolean_t	active;		/* Task has not been terminated */
+
+	/* Flags */
+	unsigned int	active:1,	/* Task has not been terminated */
+	/* boolean_t */ may_assign:1,	/* can assigned pset be changed? */
+			assign_active:1;	/* waiting for may_assign */
 
 	/* Miscellaneous */
 	vm_map_t	map;		/* Address space description */
@@ -62,8 +75,6 @@ struct task {
 	queue_head_t	thread_list;	/* list of threads */
 	int		thread_count;	/* number of threads */
 	processor_set_t	processor_set;	/* processor set for new threads */
-	boolean_t	may_assign;	/* can assigned pset be changed? */
-	boolean_t	assign_active;	/* waiting for may_assign */
 
 	/* User-visible scheduling information */
 	int		user_stop_count;	/* outstanding stops */
@@ -98,6 +109,20 @@ struct task {
 	vm_offset_t	fast_tas_base[TASK_FAST_TAS_NRAS];
 	vm_offset_t	fast_tas_end[TASK_FAST_TAS_NRAS];
 #endif	/* FAST_TAS */
+
+	/* Hardware specific data.  */
+	machine_task_t	machine;
+
+	/* Statistics */
+	natural_t	faults;		/* page faults counter */
+	natural_t	zero_fills;	/* zero fill pages counter */
+	natural_t	reactivations;	/* reactivated pages counter */
+	natural_t	pageins;	/* actual pageins couter */
+	natural_t	cow_faults;	/* copy-on-write faults counter */
+	natural_t	messages_sent;	/* messages sent counter */
+	natural_t	messages_received; /* messages received counter */
+
+	char	name[TASK_NAME_SIZE];
 };
 
 #define task_lock(task)		simple_lock(&(task)->lock)
@@ -145,21 +170,22 @@ extern kern_return_t	task_assign(
 extern kern_return_t	task_assign_default(
 	task_t		task,
 	boolean_t	assign_threads);
+extern kern_return_t	task_set_name(
+	task_t			task,
+	kernel_debug_name_t	name);
+extern void consider_task_collect(void);
 
 /*
  *	Internal only routines
  */
 
-extern void		task_init();
-extern void		task_reference();
-extern void		task_deallocate();
-extern kern_return_t	task_hold();
-extern kern_return_t	task_dowait();
-extern kern_return_t	task_release();
-extern kern_return_t	task_halt();
-
-extern kern_return_t	task_suspend_nowait();
-extern task_t		kernel_task_create();
+extern void		task_init(void);
+extern void		task_reference(task_t);
+extern void		task_deallocate(task_t);
+extern void		task_hold_locked(task_t);
+extern kern_return_t	task_hold(task_t);
+extern kern_return_t	task_dowait(task_t, boolean_t);
+extern kern_return_t	task_release(task_t);
 
 extern task_t	kernel_task;
 
