@@ -19,7 +19,7 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */
 
 #ifndef _VM_MEMORY_OBJECT_PROXY_H_
-#define _VM_MEMORY_OBJECT_PROXT_H_
+#define _VM_MEMORY_OBJECT_PROXY_H_
 
 #include <ipc/ipc_types.h>
 #include <mach/boolean.h>
@@ -30,19 +30,8 @@
 
 extern void memory_object_proxy_init (void);
 extern boolean_t memory_object_proxy_notify (mach_msg_header_t *msg);
-extern kern_return_t memory_object_create_proxy (ipc_space_t space,
-                                                 vm_prot_t max_protection,
-                                                 ipc_port_t *object,
-                                                 natural_t object_count,
-                                                 vm_offset_t *offset,
-                                                 natural_t offset_count,
-                                                 vm_offset_t *start,
-                                                 natural_t start_count,
-                                                 vm_offset_t *len,
-                                                 natural_t len_count,
-                                                 ipc_port_t *port);
 extern kern_return_t memory_object_proxy_lookup (ipc_port_t port,
                                                  ipc_port_t *object,
                                                  vm_prot_t *max_protection);
 
-#endif /* _VM_MEMORY_OBJECT_PROXT_H_ */
+#endif /* _VM_MEMORY_OBJECT_PROXY_H_ */
