@@ -47,7 +47,7 @@
 #include <kern/lock.h>
 #include <kern/queue.h>
 #include <kern/xpr.h>
-#include <kern/zalloc.h>
+#include <kern/slab.h>
 #include <vm/memory_object.h>
 #include <vm/vm_fault.h>
 #include <vm/vm_map.h>
@@ -55,13 +55,9 @@
 #include <vm/vm_page.h>
 #include <vm/vm_pageout.h>
 
-
-void memory_object_release(
-	ipc_port_t	pager,
-	pager_request_t	pager_request,
-	ipc_port_t	pager_name); /* forward */
-
-void vm_object_deactivate_pages(vm_object_t);
+#if	MACH_KDB
+#include <ddb/db_output.h>
+#endif	/* MACH_KDB */
 
 /*
  *	Virtual memory objects maintain the actual data
@@ -141,13 +137,14 @@ void vm_object_deactivate_pages(vm_object_t);
  * ZZZ Continue this comment.
  */
 
-zone_t		vm_object_zone;		/* vm backing store zone */
+struct kmem_cache	vm_object_cache; /* vm backing store cache */
 
 /*
  *	All wired-down kernel memory belongs to a single virtual
  *	memory object (kernel_object) to avoid wasting data structures.
  */
-vm_object_t	kernel_object;
+static struct vm_object	kernel_object_store;
+vm_object_t		kernel_object = &kernel_object_store;
 
 /*
  *	Virtual memory objects that are not referenced by
@@ -179,7 +176,7 @@ vm_object_t	kernel_object;
  */
 queue_head_t	vm_object_cached_list;
 int		vm_object_cached_count;
-int		vm_object_cached_max = 200;	/* may be patched*/
+int		vm_object_cached_max = 4000;	/* may be patched*/
 
 decl_simple_lock_data(,vm_object_cached_lock_data)
 
@@ -190,6 +187,15 @@ decl_simple_lock_data(,vm_object_cached_lock_data)
 #define vm_object_cache_unlock()	\
 		simple_unlock(&vm_object_cached_lock_data)
 
+/*
+ *	Number of physical pages referenced by cached objects.
+ *	This counter is protected by its own lock to work around
+ *	lock ordering issues.
+ */
+int		vm_object_cached_pages;
+
+decl_simple_lock_data(,vm_object_cached_pages_lock_data)
+
 /*
  *	Virtual memory objects are initialized from
  *	a template (see vm_object_allocate).
@@ -198,7 +204,7 @@ decl_simple_lock_data(,vm_object_cached_lock_data)
  *	object structure, be sure to add initialization
  *	(see vm_object_init).
  */
-vm_object_t	vm_object_template;
+struct vm_object	vm_object_template;
 
 /*
  *	vm_object_allocate:
@@ -206,17 +212,24 @@ vm_object_t	vm_object_template;
  *	Returns a new object with the given size.
  */
 
-vm_object_t _vm_object_allocate(
-	vm_size_t		size)
+static void _vm_object_setup(
+	vm_object_t	object,
+	vm_size_t	size)
 {
-	register vm_object_t object;
-
-	object = (vm_object_t) zalloc(vm_object_zone);
-
-	*object = *vm_object_template;
+	*object = vm_object_template;
 	queue_init(&object->memq);
 	vm_object_lock_init(object);
 	object->size = size;
+}
+
+vm_object_t _vm_object_allocate(
+	vm_size_t		size)
+{
+	vm_object_t object;
+
+	object = (vm_object_t) kmem_cache_alloc(&vm_object_cache);
+
+	_vm_object_setup(object, size);
 
 	return object;
 }
@@ -224,8 +237,8 @@ vm_object_t _vm_object_allocate(
 vm_object_t vm_object_allocate(
 	vm_size_t	size)
 {
-	register vm_object_t object;
-	register ipc_port_t port;
+	vm_object_t object;
+	ipc_port_t port;
 
 	object = _vm_object_allocate(size);
 	port = ipc_port_alloc_kernel();
@@ -244,10 +257,8 @@ vm_object_t vm_object_allocate(
  */
 void vm_object_bootstrap(void)
 {
-	vm_object_zone = zinit((vm_size_t) sizeof(struct vm_object),
-				round_page(512*1024),
-				round_page(12*1024),
-				0, "objects");
+	kmem_cache_init(&vm_object_cache, "vm_object",
+			sizeof(struct vm_object), 0, NULL, NULL, NULL, 0);
 
 	queue_init(&vm_object_cached_list);
 	simple_lock_init(&vm_object_cached_lock_data);
@@ -256,53 +267,50 @@ void vm_object_bootstrap(void)
 	 *	Fill in a template object, for quick initialization
 	 */
 
-	vm_object_template = (vm_object_t) zalloc(vm_object_zone);
-	memset(vm_object_template, 0, sizeof *vm_object_template);
+	vm_object_template.ref_count = 1;
+	vm_object_template.size = 0;
+	vm_object_template.resident_page_count = 0;
+	vm_object_template.copy = VM_OBJECT_NULL;
+	vm_object_template.shadow = VM_OBJECT_NULL;
+	vm_object_template.shadow_offset = (vm_offset_t) 0;
 
-	vm_object_template->ref_count = 1;
-	vm_object_template->size = 0;
-	vm_object_template->resident_page_count = 0;
-	vm_object_template->copy = VM_OBJECT_NULL;
-	vm_object_template->shadow = VM_OBJECT_NULL;
-	vm_object_template->shadow_offset = (vm_offset_t) 0;
+	vm_object_template.pager = IP_NULL;
+	vm_object_template.paging_offset = 0;
+	vm_object_template.pager_request = PAGER_REQUEST_NULL;
+	vm_object_template.pager_name = IP_NULL;
 
-	vm_object_template->pager = IP_NULL;
-	vm_object_template->paging_offset = 0;
-	vm_object_template->pager_request = PAGER_REQUEST_NULL;
-	vm_object_template->pager_name = IP_NULL;
+	vm_object_template.pager_created = FALSE;
+	vm_object_template.pager_initialized = FALSE;
+	vm_object_template.pager_ready = FALSE;
 
-	vm_object_template->pager_created = FALSE;
-	vm_object_template->pager_initialized = FALSE;
-	vm_object_template->pager_ready = FALSE;
-
-	vm_object_template->copy_strategy = MEMORY_OBJECT_COPY_NONE;
+	vm_object_template.copy_strategy = MEMORY_OBJECT_COPY_NONE;
 		/* ignored if temporary, will be reset before
 		 * permanent object becomes ready */
-	vm_object_template->use_shared_copy = FALSE;
-	vm_object_template->shadowed = FALSE;
+	vm_object_template.use_shared_copy = FALSE;
+	vm_object_template.shadowed = FALSE;
 
-	vm_object_template->absent_count = 0;
-	vm_object_template->all_wanted = 0; /* all bits FALSE */
+	vm_object_template.absent_count = 0;
+	vm_object_template.all_wanted = 0; /* all bits FALSE */
 
-	vm_object_template->paging_in_progress = 0;
-	vm_object_template->can_persist = FALSE;
-	vm_object_template->internal = TRUE;
-	vm_object_template->temporary = TRUE;
-	vm_object_template->alive = TRUE;
-	vm_object_template->lock_in_progress = FALSE;
-	vm_object_template->lock_restart = FALSE;
-	vm_object_template->use_old_pageout = TRUE; /* XXX change later */
-	vm_object_template->last_alloc = (vm_offset_t) 0;
+	vm_object_template.paging_in_progress = 0;
+	vm_object_template.can_persist = FALSE;
+	vm_object_template.internal = TRUE;
+	vm_object_template.temporary = TRUE;
+	vm_object_template.alive = TRUE;
+	vm_object_template.lock_in_progress = FALSE;
+	vm_object_template.lock_restart = FALSE;
+	vm_object_template.use_old_pageout = TRUE; /* XXX change later */
+	vm_object_template.last_alloc = (vm_offset_t) 0;
 
 #if	MACH_PAGEMAP
-	vm_object_template->existence_info = VM_EXTERNAL_NULL;
+	vm_object_template.existence_info = VM_EXTERNAL_NULL;
 #endif	/* MACH_PAGEMAP */
 
 		/*
 	 *	Initialize the "kernel object"
 	 */
 
-	kernel_object = _vm_object_allocate(
+	_vm_object_setup(kernel_object,
 		VM_MAX_KERNEL_ADDRESS - VM_MIN_KERNEL_ADDRESS);
 
 	/*
@@ -310,7 +318,7 @@ void vm_object_bootstrap(void)
 	 *	kernel object so that no limit is imposed on submap sizes.
 	 */
 
-	vm_submap_object = _vm_object_allocate(
+	_vm_object_setup(vm_submap_object,
 		VM_MAX_KERNEL_ADDRESS - VM_MIN_KERNEL_ADDRESS);
 
 #if	MACH_PAGEMAP
@@ -337,7 +345,7 @@ void vm_object_init(void)
  *	Gets another reference to the given object.
  */
 void vm_object_reference(
-	register vm_object_t	object)
+	vm_object_t	object)
 {
 	if (object == VM_OBJECT_NULL)
 		return;
@@ -360,7 +368,7 @@ void vm_object_reference(
  *	No object may be locked.
  */
 void vm_object_deallocate(
-	register vm_object_t	object)
+	vm_object_t	object)
 {
 	vm_object_t	temp;
 
@@ -407,6 +415,7 @@ void vm_object_deallocate(
 			queue_enter(&vm_object_cached_list, object,
 				vm_object_t, cached_list);
 			overflow = (++vm_object_cached_count > vm_object_cached_max);
+			vm_object_cached_pages_update(object->resident_page_count);
 			vm_object_cache_unlock();
 
 			vm_object_deactivate_pages(object);
@@ -513,8 +522,6 @@ void vm_object_deallocate(
 	}
 }
 
-boolean_t	vm_object_terminate_remove_all = FALSE;
-
 /*
  *	Routine:	vm_object_terminate
  *	Purpose:
@@ -529,10 +536,10 @@ boolean_t	vm_object_terminate_remove_all = FALSE;
  *		object will cease to exist.
  */
 void vm_object_terminate(
-	register vm_object_t	object)
+	vm_object_t	object)
 {
-	register vm_page_t	p;
-	vm_object_t		shadow_object;
+	vm_page_t	p;
+	vm_object_t	shadow_object;
 
 	/*
 	 *	Make sure the object isn't already being terminated
@@ -660,7 +667,7 @@ void vm_object_terminate(
 	 *	Free the space for the object.
 	 */
 
-	zfree(vm_object_zone, (vm_offset_t) object);
+	kmem_cache_free(&vm_object_cache, (vm_offset_t) object);
 }
 
 /*
@@ -735,7 +742,6 @@ void memory_object_release(
 void vm_object_abort_activity(
 	vm_object_t	object)
 {
-	register
 	vm_page_t	p;
 	vm_page_t	next;
 
@@ -789,17 +795,12 @@ void vm_object_abort_activity(
  *		or from port destruction handling (via vm_object_destroy).
  */
 kern_return_t memory_object_destroy(
-	register
 	vm_object_t	object,
 	kern_return_t	reason)
 {
 	ipc_port_t	old_object,  old_name;
 	pager_request_t	old_control;
 
-#ifdef	lint
-	reason++;
-#endif	/* lint */
-
 	if (object == VM_OBJECT_NULL)
 		return KERN_SUCCESS;
 
@@ -875,9 +876,9 @@ kern_return_t memory_object_destroy(
  *	The object must be locked.
  */
 void vm_object_deactivate_pages(
-	register vm_object_t	object)
+	vm_object_t	object)
 {
-	register vm_page_t	p;
+	vm_page_t	p;
 
 	queue_iterate(&object->memq, p, vm_page_t, listq) {
 		vm_page_lock_queues();
@@ -914,9 +915,9 @@ void vm_object_deactivate_pages(
 boolean_t vm_object_pmap_protect_by_page = FALSE;
 
 void vm_object_pmap_protect(
-	register vm_object_t	object,
-	register vm_offset_t	offset,
-	vm_offset_t		size,
+	vm_object_t		object,
+	vm_offset_t		offset,
+	vm_size_t		size,
 	pmap_t			pmap,
 	vm_offset_t		pmap_start,
 	vm_prot_t		prot)
@@ -937,8 +938,8 @@ void vm_object_pmap_protect(
 	    }
 
 	    {
-		register vm_page_t	p;
-		register vm_offset_t	end;
+		vm_page_t	p;
+		vm_offset_t	end;
 
 		end = offset + size;
 
@@ -969,7 +970,7 @@ void vm_object_pmap_protect(
 		 * Must follow shadow chain to remove access
 		 * to pages in shadowed objects.
 		 */
-		register vm_object_t	next_object;
+		vm_object_t	next_object;
 
 		next_object = object->shadow;
 		if (next_object != VM_OBJECT_NULL) {
@@ -1006,11 +1007,11 @@ void vm_object_pmap_protect(
  *	The object must *not* be locked.
  */
 void vm_object_pmap_remove(
-	register vm_object_t	object,
-	register vm_offset_t	start,
-	register vm_offset_t	end)
+	vm_object_t	object,
+	vm_offset_t	start,
+	vm_offset_t	end)
 {
-	register vm_page_t	p;
+	vm_page_t	p;
 
 	if (object == VM_OBJECT_NULL)
 		return;
@@ -1056,7 +1057,6 @@ void vm_object_pmap_remove(
  *		VM_OBJECT_NULL.
  */
 kern_return_t vm_object_copy_slowly(
-	register
 	vm_object_t	src_object,
 	vm_offset_t	src_offset,
 	vm_size_t	size,
@@ -1110,7 +1110,6 @@ kern_return_t vm_object_copy_slowly(
 			vm_prot_t	prot = VM_PROT_READ;
 			vm_page_t	_result_page;
 			vm_page_t	top_page;
-			register
 			vm_page_t	result_page;
 
 			vm_object_lock(src_object);
@@ -1230,8 +1229,6 @@ kern_return_t vm_object_copy_slowly(
  *		The object should be unlocked on entry and exit.
  */
 
-vm_object_t	vm_object_copy_delayed();	/* forward declaration */
-
 boolean_t vm_object_copy_temporary(
 	vm_object_t	*_object,		/* INOUT */
 	vm_offset_t	*_offset,		/* INOUT */
@@ -1240,10 +1237,6 @@ boolean_t vm_object_copy_temporary(
 {
 	vm_object_t	object = *_object;
 
-#ifdef	lint
-	++*_offset;
-#endif	/* lint */
-
 	if (object == VM_OBJECT_NULL) {
 		*_src_needs_copy = FALSE;
 		*_dst_needs_copy = FALSE;
@@ -1473,7 +1466,7 @@ vm_object_t vm_object_copy_delayed(
 	 *		 synchronization required in the "push"
 	 *		 operation described above.
 	 *
-	 *	The copy-on-write is said to be assymetric because
+	 *	The copy-on-write is said to be asymmetric because
 	 *	the original object is *not* marked copy-on-write.
 	 *	A copied page is pushed to the copy object, regardless
 	 *	which party attempted to modify the page.
@@ -1606,7 +1599,6 @@ vm_object_t vm_object_copy_delayed(
  *		and may be interrupted.
  */
 kern_return_t	vm_object_copy_strategically(
-	register
 	vm_object_t	src_object,
 	vm_offset_t	src_offset,
 	vm_size_t	size,
@@ -1719,8 +1711,8 @@ void vm_object_shadow(
 	vm_offset_t	*offset,	/* IN/OUT */
 	vm_size_t	length)
 {
-	register vm_object_t	source;
-	register vm_object_t	result;
+	vm_object_t	source;
+	vm_object_t	result;
 
 	source = *object;
 
@@ -1857,6 +1849,7 @@ vm_object_t vm_object_lookup(
 				queue_remove(&vm_object_cached_list, object,
 					     vm_object_t, cached_list);
 				vm_object_cached_count--;
+				vm_object_cached_pages_update(-object->resident_page_count);
 			}
 
 			object->ref_count++;
@@ -1888,6 +1881,7 @@ vm_object_t vm_object_lookup_name(
 				queue_remove(&vm_object_cached_list, object,
 					     vm_object_t, cached_list);
 				vm_object_cached_count--;
+				vm_object_cached_pages_update(-object->resident_page_count);
 			}
 
 			object->ref_count++;
@@ -1924,6 +1918,7 @@ void vm_object_destroy(
 		queue_remove(&vm_object_cached_list, object,
 				vm_object_t, cached_list);
 		vm_object_cached_count--;
+		vm_object_cached_pages_update(-object->resident_page_count);
 	}
 	object->ref_count++;
 
@@ -1989,7 +1984,6 @@ vm_object_t vm_object_enter(
 	vm_size_t	size,
 	boolean_t	internal)
 {
-	register
 	vm_object_t	object;
 	vm_object_t	new_object;
 	boolean_t	must_init;
@@ -2077,6 +2071,7 @@ restart:
 			queue_remove(&vm_object_cached_list, object,
 					vm_object_t, cached_list);
 			vm_object_cached_count--;
+			vm_object_cached_pages_update(-object->resident_page_count);
 		}
 		object->ref_count++;
 		vm_object_unlock(object);
@@ -2206,7 +2201,6 @@ restart:
  *		daemon will be using this routine.
  */
 void vm_object_pager_create(
-	register
 	vm_object_t	object)
 {
 	ipc_port_t	pager;
@@ -2351,14 +2345,14 @@ boolean_t	vm_object_collapse_bypass_allowed = TRUE;
  *	so the caller should hold a reference for the object.
  */
 void vm_object_collapse(
-	register vm_object_t	object)
+	vm_object_t	object)
 {
-	register vm_object_t	backing_object;
-	register vm_offset_t	backing_offset;
-	register vm_size_t	size;
-	register vm_offset_t	new_offset;
-	register vm_page_t	p, pp;
-	ipc_port_t old_name_port;
+	vm_object_t	backing_object;
+	vm_offset_t	backing_offset;
+	vm_size_t	size;
+	vm_offset_t	new_offset;
+	vm_page_t	p, pp;
+	ipc_port_t 	old_name_port;
 
 	if (!vm_object_collapse_allowed)
 		return;
@@ -2471,9 +2465,7 @@ void vm_object_collapse(
 
 				if (p->offset < backing_offset ||
 				    new_offset >= size) {
-					vm_page_lock_queues();
-					vm_page_free(p);
-					vm_page_unlock_queues();
+					VM_PAGE_FREE(p);
 				} else {
 				    pp = vm_page_lookup(object, new_offset);
 				    if (pp != VM_PAGE_NULL && !pp->absent) {
@@ -2482,9 +2474,7 @@ void vm_object_collapse(
 					 *	Throw away the backing object's
 					 *	page.
 					 */
-					vm_page_lock_queues();
-					vm_page_free(p);
-					vm_page_unlock_queues();
+					VM_PAGE_FREE(p);
 				    }
 				    else {
 					if (pp != VM_PAGE_NULL) {
@@ -2508,9 +2498,7 @@ void vm_object_collapse(
 					     */
 					    panic("vm_object_collapse: bad case");
 
-					    vm_page_lock_queues();
-					    vm_page_free(pp);
-					    vm_page_unlock_queues();
+					    VM_PAGE_FREE(pp);
 
 					    /*
 					     *	Fall through to move the backing
@@ -2545,11 +2533,11 @@ void vm_object_collapse(
 				/* Fall through to... */
 
 			    default:
-				printf("vm_object_collapse: %#x (pager %#x, request %#x) up to %#x\n",
+				printf("vm_object_collapse: %p (pager %p, request %p) up to %p\n",
 					backing_object, backing_object->pager, backing_object->pager_request,
 					object);
 				if (vm_object_collapse_debug > 2)
-				    Debugger("vm_object_collapse");
+				    SoftDebugger("vm_object_collapse");
 			}
 
 			object->pager = backing_object->pager;
@@ -2624,7 +2612,7 @@ void vm_object_collapse(
 			vm_object_unlock(object);
 			if (old_name_port != IP_NULL)
 				ipc_port_dealloc_kernel(old_name_port);
-			zfree(vm_object_zone, (vm_offset_t) backing_object);
+			kmem_cache_free(&vm_object_cache, (vm_offset_t) backing_object);
 			vm_object_lock(object);
 
 			object_collapses++;
@@ -2734,11 +2722,11 @@ unsigned int vm_object_page_remove_lookup = 0;
 unsigned int vm_object_page_remove_iterate = 0;
 
 void vm_object_page_remove(
-	register vm_object_t	object,
-	register vm_offset_t	start,
-	register vm_offset_t	end)
+	vm_object_t	object,
+	vm_offset_t	start,
+	vm_offset_t	end)
 {
-	register vm_page_t	p, next;
+	vm_page_t	p, next;
 
 	/*
 	 *	One and two page removals are most popular.
@@ -2755,9 +2743,7 @@ void vm_object_page_remove(
 				if (!p->fictitious)
 					pmap_page_protect(p->phys_addr,
 							  VM_PROT_NONE);
-				vm_page_lock_queues();
-				vm_page_free(p);
-				vm_page_unlock_queues();
+				VM_PAGE_FREE(p);
 			}
 		}
 	} else {
@@ -2770,9 +2756,7 @@ void vm_object_page_remove(
 				if (!p->fictitious)
 				    pmap_page_protect(p->phys_addr,
 						      VM_PROT_NONE);
-				vm_page_lock_queues();
-				vm_page_free(p);
-				vm_page_unlock_queues();
+				VM_PAGE_FREE(p);
 			}
 			p = next;
 		}
@@ -2803,7 +2787,7 @@ void vm_object_page_remove(
  */
 
 boolean_t vm_object_coalesce(
-	register vm_object_t prev_object,
+	vm_object_t 	prev_object,
 	vm_object_t	next_object,
 	vm_offset_t	prev_offset,
 	vm_offset_t	next_offset,
@@ -2812,10 +2796,6 @@ boolean_t vm_object_coalesce(
 {
 	vm_size_t	newsize;
 
-#ifdef	lint
-	next_offset++;
-#endif	/* lint */
-
 	if (next_object != VM_OBJECT_NULL) {
 		return FALSE;
 	}
@@ -2941,9 +2921,7 @@ vm_object_page_map(
 	    if ((old_page = vm_page_lookup(object, offset))
 			!= VM_PAGE_NULL)
 	    {
-		vm_page_lock_queues();
-		vm_page_free(old_page);
-		vm_page_unlock_queues();
+		VM_PAGE_FREE(old_page);
 	    }
 
 	    vm_page_init(m, addr);
@@ -2960,6 +2938,7 @@ vm_object_page_map(
 
 
 #if	MACH_KDB
+#include <vm/vm_print.h>
 #define printf	kdbprintf
 
 boolean_t	vm_object_print_pages = FALSE;
@@ -2970,9 +2949,9 @@ boolean_t	vm_object_print_pages = FALSE;
 void vm_object_print(
 	vm_object_t	object)
 {
-	register vm_page_t	p;
+	vm_page_t	p;
 
-	register int count;
+	int 		count;
 
 	if (object == VM_OBJECT_NULL)
 		return;
