@@ -27,7 +27,7 @@
  * the rights to redistribute these changes.
  */
 /*
- *	File:	vm/vm_page.c
+ *	File:	vm/vm_resident.c
  *	Author:	Avadis Tevanian, Jr., Michael Wayne Young
  *
  *	Resident memory management module.
@@ -45,7 +45,7 @@
 #include <mach/vm_statistics.h>
 #include <machine/vm_param.h>
 #include <kern/xpr.h>
-#include <kern/zalloc.h>
+#include <kern/slab.h>
 #include <vm/pmap.h>
 #include <vm/vm_map.h>
 #include <vm/vm_page.h>
@@ -58,12 +58,14 @@
 #include <vm/vm_user.h>
 #endif
 
-/* in zalloc.c XXX */
-extern vm_offset_t	zdata;
-extern vm_size_t	zdata_size;
+#if	MACH_KDB
+#include <ddb/db_output.h>
+#include <vm/vm_print.h>
+#endif	/* MACH_KDB */
+
 
 /*
- *	Associated with eacn page of user-allocatable memory is a
+ *	Associated with each page of user-allocatable memory is a
  *	page structure.
  */
 
@@ -126,7 +128,7 @@ unsigned int	vm_page_free_count_minimum;	/* debugging */
  *	These page structures are allocated the way
  *	most other kernel structures are.
  */
-zone_t	vm_page_zone;
+struct kmem_cache	vm_page_cache;
 
 /*
  *	Fictitious pages don't have a physical address,
@@ -190,7 +192,7 @@ void vm_page_bootstrap(
 	vm_offset_t *startp,
 	vm_offset_t *endp)
 {
-	register vm_page_t m;
+	vm_page_t m;
 	int i;
 
 	/*
@@ -239,14 +241,11 @@ void vm_page_bootstrap(
 	vm_page_free_wanted = 0;
 
 	/*
-	 *	Steal memory for the zone system.
+	 *	Steal memory for the kernel map entries.
 	 */
 
-	kentry_data_size = kentry_count * sizeof(struct vm_map_entry);
 	kentry_data = pmap_steal_memory(kentry_data_size);
 
-	zdata = pmap_steal_memory(zdata_size);
-
 	/*
 	 *	Allocate (and initialize) the virtual-to-physical
 	 *	table hash buckets.
@@ -275,7 +274,7 @@ void vm_page_bootstrap(
 				  sizeof(vm_page_bucket_t));
 
 	for (i = 0; i < vm_page_bucket_count; i++) {
-		register vm_page_bucket_t *bucket = &vm_page_buckets[i];
+		vm_page_bucket_t *bucket = &vm_page_buckets[i];
 
 		bucket->pages = VM_PAGE_NULL;
 		simple_lock_init(&bucket->lock);
@@ -393,6 +392,11 @@ void pmap_startup(
 		vm_page_init(&pages[i], paddr);
 		pages_initialized++;
 	}
+	i = 0;
+	while (pmap_next_page(&paddr))
+		i++;
+	if (i)
+		printf("%u memory page(s) left away\n", i);
 
 	/*
 	 * Release pages in reverse order so that physical pages
@@ -425,10 +429,8 @@ void pmap_startup(
  */
 void		vm_page_module_init(void)
 {
-	vm_page_zone = zinit((vm_size_t) sizeof(struct vm_page),
-			     VM_MAX_KERNEL_ADDRESS - VM_MIN_KERNEL_ADDRESS,
-			     PAGE_SIZE,
-			     0, "vm pages");
+	kmem_cache_init(&vm_page_cache, "vm_page", sizeof(struct vm_page), 0,
+			NULL, NULL, NULL, 0);
 }
 
 /*
@@ -450,7 +452,7 @@ void vm_page_create(
 	for (paddr = round_page(start);
 	     paddr < trunc_page(end);
 	     paddr += PAGE_SIZE) {
-		m = (vm_page_t) zalloc(vm_page_zone);
+		m = (vm_page_t) kmem_cache_alloc(&vm_page_cache);
 		if (m == VM_PAGE_NULL)
 			panic("vm_page_create");
 
@@ -481,11 +483,11 @@ void vm_page_create(
  */
 
 void vm_page_insert(
-	register vm_page_t	mem,
-	register vm_object_t	object,
-	register vm_offset_t	offset)
+	vm_page_t	mem,
+	vm_object_t	object,
+	vm_offset_t	offset)
 {
-	register vm_page_bucket_t *bucket;
+	vm_page_bucket_t *bucket;
 
 	VM_PAGE_CHECK(mem);
 
@@ -521,6 +523,10 @@ void vm_page_insert(
 	 */
 
 	object->resident_page_count++;
+	assert(object->resident_page_count >= 0);
+
+	if (object->can_persist && (object->ref_count == 0))
+		vm_object_cached_pages_update(1);
 
 	/*
 	 *	Detect sequential access and inactivate previous page.
@@ -549,11 +555,11 @@ void vm_page_insert(
  */
 
 void vm_page_replace(
-	register vm_page_t	mem,
-	register vm_object_t	object,
-	register vm_offset_t	offset)
+	vm_page_t	mem,
+	vm_object_t	object,
+	vm_offset_t	offset)
 {
-	register vm_page_bucket_t *bucket;
+	vm_page_bucket_t *bucket;
 
 	VM_PAGE_CHECK(mem);
 
@@ -576,7 +582,7 @@ void vm_page_replace(
 	simple_lock(&bucket->lock);
 	if (bucket->pages) {
 		vm_page_t *mp = &bucket->pages;
-		register vm_page_t m = *mp;
+		vm_page_t m = *mp;
 		do {
 			if (m->object == object && m->offset == offset) {
 				/*
@@ -589,6 +595,10 @@ void vm_page_replace(
 				m->tabled = FALSE;
 				object->resident_page_count--;
 
+				if (object->can_persist
+				    && (object->ref_count == 0))
+					vm_object_cached_pages_update(-1);
+
 				/*
 				 * Return page to the free list.
 				 * Note the page is not tabled now, so this
@@ -620,6 +630,10 @@ void vm_page_replace(
 	 */
 
 	object->resident_page_count++;
+	assert(object->resident_page_count >= 0);
+
+	if (object->can_persist && (object->ref_count == 0))
+		vm_object_cached_pages_update(1);
 }
 
 /*
@@ -632,10 +646,10 @@ void vm_page_replace(
  */
 
 void vm_page_remove(
-	register vm_page_t	mem)
+	vm_page_t		mem)
 {
-	register vm_page_bucket_t	*bucket;
-	register vm_page_t	this;
+	vm_page_bucket_t	*bucket;
+	vm_page_t		this;
 
 	assert(mem->tabled);
 	VM_PAGE_CHECK(mem);
@@ -651,7 +665,7 @@ void vm_page_remove(
 
 		bucket->pages = mem->next;
 	} else {
-		register vm_page_t	*prev;
+		vm_page_t	*prev;
 
 		for (prev = &this->next;
 		     (this = *prev) != mem;
@@ -675,6 +689,9 @@ void vm_page_remove(
 	mem->object->resident_page_count--;
 
 	mem->tabled = FALSE;
+
+	if (mem->object->can_persist && (mem->object->ref_count == 0))
+		vm_object_cached_pages_update(-1);
 }
 
 /*
@@ -687,11 +704,11 @@ void vm_page_remove(
  */
 
 vm_page_t vm_page_lookup(
-	register vm_object_t	object,
-	register vm_offset_t	offset)
+	vm_object_t		object,
+	vm_offset_t		offset)
 {
-	register vm_page_t	mem;
-	register vm_page_bucket_t *bucket;
+	vm_page_t		mem;
+	vm_page_bucket_t 	*bucket;
 
 	/*
 	 *	Search the hash table for this object/offset pair
@@ -718,9 +735,9 @@ vm_page_t vm_page_lookup(
  *	The object must be locked.
  */
 void vm_page_rename(
-	register vm_page_t	mem,
-	register vm_object_t	new_object,
-	vm_offset_t		new_offset)
+	vm_page_t	mem,
+	vm_object_t	new_object,
+	vm_offset_t	new_offset)
 {
 	/*
 	 *	Changes to mem->object require the page lock because
@@ -757,7 +774,7 @@ void vm_page_init(
 
 vm_page_t vm_page_grab_fictitious(void)
 {
-	register vm_page_t m;
+	vm_page_t m;
 
 	simple_lock(&vm_page_queue_free_lock);
 	m = vm_page_queue_fictitious;
@@ -778,7 +795,7 @@ vm_page_t vm_page_grab_fictitious(void)
  */
 
 void vm_page_release_fictitious(
-	register vm_page_t m)
+	vm_page_t m)
 {
 	simple_lock(&vm_page_queue_free_lock);
 	if (m->free)
@@ -801,11 +818,11 @@ int vm_page_fictitious_quantum = 5;
 
 void vm_page_more_fictitious(void)
 {
-	register vm_page_t m;
+	vm_page_t m;
 	int i;
 
 	for (i = 0; i < vm_page_fictitious_quantum; i++) {
-		m = (vm_page_t) zalloc(vm_page_zone);
+		m = (vm_page_t) kmem_cache_alloc(&vm_page_cache);
 		if (m == VM_PAGE_NULL)
 			panic("vm_page_more_fictitious");
 
@@ -822,10 +839,10 @@ void vm_page_more_fictitious(void)
  */
 
 boolean_t vm_page_convert(
-	register vm_page_t m,
+	vm_page_t m,
 	boolean_t external)
 {
-	register vm_page_t real_m;
+	vm_page_t real_m;
 
 	real_m = vm_page_grab(external);
 	if (real_m == VM_PAGE_NULL)
@@ -851,7 +868,7 @@ boolean_t vm_page_convert(
 vm_page_t vm_page_grab(
 	boolean_t external)
 {
-	register vm_page_t	mem;
+	vm_page_t	mem;
 
 	simple_lock(&vm_page_queue_free_lock);
 
@@ -901,7 +918,7 @@ vm_page_t vm_page_grab(
 	return mem;
 }
 
-vm_offset_t vm_page_grab_phys_addr()
+vm_offset_t vm_page_grab_phys_addr(void)
 {
 	vm_page_t p = vm_page_grab(FALSE);
 	if (p == VM_PAGE_NULL)
@@ -932,10 +949,10 @@ vm_page_grab_contiguous_pages(
 	natural_t	*bits,
 	boolean_t	external)
 {
-	register int	first_set;
+	int		first_set;
 	int		size, alloc_size;
 	kern_return_t	ret;
-	vm_page_t       mem, prevmem;
+	vm_page_t       mem, *prevmemp;
 
 #ifndef	NBBY
 #define	NBBY	8	/* size in bits of sizeof()`s unity */
@@ -976,6 +993,7 @@ vm_page_grab_contiguous_pages(
 
 	if ((vm_page_free_count < vm_page_free_reserved)
 	    || (vm_page_external_count >= vm_page_external_limit)) {
+		printf_once("no more room for vm_page_grab_contiguous_pages");
 		simple_unlock(&vm_page_queue_free_lock);
 		return KERN_RESOURCE_SHORTAGE;
 	}
@@ -988,7 +1006,7 @@ vm_page_grab_contiguous_pages(
 	 */
 	mem = vm_page_queue_free;
 	while (mem) {
-		register int word_index, bit_index;
+		int word_index, bit_index;
 
 		bit_index = (mem->phys_addr >> PAGE_SHIFT);
 		word_index = bit_index / NBPEL;
@@ -1005,14 +1023,14 @@ vm_page_grab_contiguous_pages(
 	 *	the free list.
 	 */
 	{
-	    register int	bits_so_far = 0, i;
+	    int	bits_so_far = 0, i;
 
 		first_set = 0;
 
 		for (i = 0; i < size; i += sizeof(natural_t)) {
 
-		    register natural_t	v = bits[i / sizeof(natural_t)];
-		    register int	bitpos;
+		    natural_t	v = bits[i / sizeof(natural_t)];
+		    int		bitpos;
 
 		    /*
 		     * Bitscan this one word
@@ -1043,7 +1061,6 @@ count_ones:
 			 * search for beans here
 			 */
 			bits_so_far = 0;
-count_zeroes:
 			while ((bitpos < NBPEL) && ((v & 1) == 0)) {
 			    bitpos++;
 			    v >>= 1;
@@ -1063,9 +1080,9 @@ count_zeroes:
 	/*
 	 *	We could not find enough contiguous pages.
 	 */
-not_found_em:
 	simple_unlock(&vm_page_queue_free_lock);
 
+	printf_once("no contiguous room for vm_page_grab_contiguous_pages");
 	ret = KERN_RESOURCE_SHORTAGE;
 	goto out;
 
@@ -1082,7 +1099,7 @@ found_em:
 	if (external)
 		vm_page_external_count += npages;
 	{
-	    register vm_offset_t	first_phys, last_phys;
+	    vm_offset_t	first_phys, last_phys;
 
 	    /* cache values for compare */
 	    first_phys = first_set << PAGE_SHIFT;
@@ -1090,18 +1107,17 @@ found_em:
 
 	    /* running pointers */
 	    mem = vm_page_queue_free;
-	    prevmem = VM_PAGE_NULL;
+	    prevmemp = &vm_page_queue_free;
 
 	    while (mem) {
 
-		register vm_offset_t	addr;
+		vm_offset_t	addr;
 
 		addr = mem->phys_addr;
 
 		if ((addr >= first_phys) &&
 		    (addr <  last_phys)) {
-		    if (prevmem)
-			prevmem->pageq.next = mem->pageq.next;
+		    *prevmemp = (vm_page_t) mem->pageq.next;
 		    pages[(addr - first_phys) >> PAGE_SHIFT] = mem;
 		    mem->free = FALSE;
 		    mem->extcounted = mem->external = external;
@@ -1110,7 +1126,7 @@ found_em:
 		     */
 		    if (--npages == 0) break;
 		} else
-		    prevmem = mem;
+		    prevmemp = (vm_page_t *) &mem->pageq.next;
 
 		mem = (vm_page_t) mem->pageq.next;
 	    }
@@ -1149,8 +1165,8 @@ out:
  */
 
 void vm_page_release(
-	register vm_page_t	mem,
-	boolean_t external)
+	vm_page_t	mem,
+	boolean_t 	external)
 {
 	simple_lock(&vm_page_queue_free_lock);
 	if (mem->free)
@@ -1202,10 +1218,6 @@ void vm_page_wait(
 	void (*continuation)(void))
 {
 
-#ifndef CONTINUATIONS
-	assert (continuation == 0);
-#endif
-
 	/*
 	 *	We can't use vm_page_free_reserved to make this
 	 *	determination.  Consider: some thread might
@@ -1245,7 +1257,7 @@ vm_page_t vm_page_alloc(
 	vm_object_t	object,
 	vm_offset_t	offset)
 {
-	register vm_page_t	mem;
+	vm_page_t	mem;
 
 	mem = vm_page_grab(!object->internal);
 	if (mem == VM_PAGE_NULL)
@@ -1267,7 +1279,7 @@ vm_page_t vm_page_alloc(
  *	Object and page queues must be locked prior to entry.
  */
 void vm_page_free(
-	register vm_page_t	mem)
+	vm_page_t	mem)
 {
 	if (mem->free)
 		panic("vm_page_free");
@@ -1318,7 +1330,7 @@ void vm_page_free(
  *	The page's object and the page queues must be locked.
  */
 void vm_page_wire(
-	register vm_page_t	mem)
+	vm_page_t	mem)
 {
 	VM_PAGE_CHECK(mem);
 
@@ -1339,7 +1351,7 @@ void vm_page_wire(
  *	The page's object and the page queues must be locked.
  */
 void vm_page_unwire(
-	register vm_page_t	mem)
+	vm_page_t	mem)
 {
 	VM_PAGE_CHECK(mem);
 
@@ -1362,7 +1374,7 @@ void vm_page_unwire(
  *	The page queues must be locked.
  */
 void vm_page_deactivate(
-	register vm_page_t	m)
+	vm_page_t	m)
 {
 	VM_PAGE_CHECK(m);
 
@@ -1396,7 +1408,7 @@ void vm_page_deactivate(
  */
 
 void vm_page_activate(
-	register vm_page_t	m)
+	vm_page_t	m)
 {
 	VM_PAGE_CHECK(m);
 
@@ -1493,10 +1505,10 @@ vm_page_info(
  *	Routine:	vm_page_print [exported]
  */
 void		vm_page_print(p)
-	vm_page_t	p;
+	const vm_page_t	p;
 {
 	iprintf("Page 0x%X: object 0x%X,", (vm_offset_t) p, (vm_offset_t) p->object);
-	 printf(" offset 0x%X", (vm_offset_t) p->offset);
+	 printf(" offset 0x%X", p->offset);
 	 printf("wire_count %d,", p->wire_count);
 	 printf(" %s",
 		(p->active ? "active" : (p->inactive ? "inactive" : "loose")));
@@ -1521,7 +1533,7 @@ void		vm_page_print(p)
 	 printf("%s,",
 		(p->tabled ? "" : "not_tabled"));
 	 printf("phys_addr = 0x%X, lock = 0x%X, unlock_request = 0x%X\n",
-	 	(vm_offset_t) p->phys_addr,
+		p->phys_addr,
 		(vm_offset_t) p->page_lock,
 		(vm_offset_t) p->unlock_request);
 }
